/*
   $Header: /u/drspeech/repos/rtst/rtst_fail.c,v 1.14 1996/10/15 22:11:36 davidj Exp $

   A selection of tests of "rtst", all of which should fail if everything
   works as expected!
*/

#include <stdio.h>
#include "rtst.h"

void
padvec_fail_start()
{
    /* Allocate a vector. */
    unsigned char *p = (unsigned char*) rtst_padvec_new(123, 1);

    /* Can't guarantee that one byte will fail, since might match signature */
    /* byte. */
    p[-3] = 3;
    p[-1] = 2;

    rtst_padvec_del(p);
}

void
padvec_fail_end()
{
    /* Allocate a vector. */
    unsigned char *p = (unsigned char*) rtst_padvec_new(123, 1);

    /* Can't guarantee that one byte will fail, since might match signature */
    /* byte. */
    p[129] = 3;
    p[124] = 2;

    rtst_padvec_del(p);
}

void
padvec_fail_both()
{
    /* Allocate a vector. */
    unsigned char *p = (unsigned char*) rtst_padvec_new(123, 1);

    /* Can't guarantee that one byte will fail, since might match signature */
    /* byte. */
    p[-3] = 3;
    p[124] = 2;

    rtst_padvec_del(p);
}

void
padmat_fail_start()
{
    /* Allocate a matrix. */
    unsigned char *p = (unsigned char*) rtst_padmat_new(12, 13, 15, 1);

    /* Can't guarantee that one byte will fail, since might match signature */
    /* byte. */
    p[-3] = 3;
    p[-1] = 2;

    rtst_padmat_del(p);
}

void
padmat_fail_end()
{
    /* Allocate a matrix. */
    unsigned char *p = (unsigned char*) rtst_padmat_new(12, 13, 15, 1);

    /* Can't guarantee that one byte will fail, since might match signature */
    /* byte. */
    p[12*15] = 3;
    p[12*15+1] = 2;

    rtst_padmat_del(p);
}

/* Check writing past and of row. */
void
padmat_fail_row()
{
    /* Allocate a matrix. */
    unsigned char *p = (unsigned char*) rtst_padmat_new(12, 13, 15, 1);

    /* Can't guarantee that one byte will fail, since might match signature */
    /* byte. */
    p[13] = 3;
    p[15+13] = 2;

    rtst_padmat_del(p);
}

void
checknear_fail()
{
    float a[2] = { 0.0, 1.0 };
    float b[2] = { 0.0, -1.0 };
    
    rtst_checknear_fvfvf(2, 1.99f, a, b);
}

void
checkeq_fail()
{
    /* 0.0/0.0 gives NaN */
    const float NaN_f = rtst_ieee_divide_ff_f(0.0f, 0.0f);

    /* 1.0/0.0 gives +Inf */
    const double Inf_f = rtst_ieee_divide_ff_f(1.0f, 0.0f);

    /* Check that NaNs are not considered the same as anything else */
    float a[2] = { 0.0, 0.0 };
    float b[2] = { 0.0, 0.0 };

    a[1] = NaN_f;
    b[1] = Inf_f;
    
    rtst_checkeq_vfvf(2, a, b);
}

void
checkrange_fail()
{
    float a[2] = { -0.26, 0.92 };
    
    rtst_checkrange_ffvf(2, -0.7, 0.81, a);
}

int
main(int argc, char* argv[])
{
    int arg;

    arg = rtst_args(argc, argv);
    rtst_start("rtst_fail");
    if (arg<argc)
    {
	switch(*argv[arg])
	{
	case '0':
	    rtst_assert(1==0);
	    /* This will never be executed - it is only here to check the
	       syntax works. */
	    (rtst_assert(1==0));
	    break;
	case '1':
	    rtst_failed("failed - arg 1");
	    break;
        case '2':
            padvec_fail_start();
            break;
        case '3':
            padvec_fail_end();
            break;
        case '4':
            padvec_fail_both();
            break;
	case '5':
	    checknear_fail();
	    break;
	case '6':
	    checkeq_fail();
	    break;
	case '7':
	    checkrange_fail();
	    break;
        case '8':
            padmat_fail_start();
            break;
        case '9':
            padmat_fail_end();
            break;
        case 'a':
            padmat_fail_row();
            break;
	default:
	    rtst_log("***** THIS MESSAGE INDICATES TEST FAILURE - WE SHOULD NOT HAVE PASSED *****\n");
	    rtst_passed();
	}
    }
    else
	rtst_failed("failed - no arg");

    return 0;
}
