const char* rtst_badrand_rcsid = "$Header: /u/drspeech/repos/rtst/rtst_rand.c,v 1.14 1996/11/25 20:09:37 krste Exp $";
/*
**
** rtst - various routines useful for writing test programs
**
** This file contains the various random number generator functions provided.
** All of these use the rand48 generator with the state held in rtst_seed48.
*/

#include <assert.h>

#include "rtst.h"

/* Not all systems supply prototypes for these functions. */
extern double drand48();
extern double erand48(unsigned short xsubi[3]);
extern long lrand48();
extern long nrand48(unsigned short xsubi[3]);
extern long mrand48();
extern long jrand48(unsigned short xsubi[3]);
extern void srand48(long seedval);
extern unsigned short *seed48(const unsigned short seed16v[3]);
extern void lcong48(const unsigned short param[7]);

/***********************************************************************
** Functions that provide a uniform distribution of random values.
** Note that the limits are _INCLUSIVE_.
*/

rtst_int8
rtst_urand_i8i8_i8(rtst_int8 min, rtst_int8 max)
{
    rtst_int8 res;
    rtst_int32 range;

    assert(max>=min);
    range = (rtst_int32) max - (rtst_int32) min + 1;
    res = (rtst_int8) ((nrand48(rtst_state48) % range) + min);
    return res;
}

rtst_uint8
rtst_urand_u8u8_u8(rtst_uint8 min, rtst_uint8 max)
{
    rtst_uint8 res;
    rtst_int32 range;

    assert(max>=min);
    range = (rtst_int32) max - (rtst_int32) min + 1;
    res = (rtst_uint8) ((nrand48(rtst_state48) % range) + min);
    return res;
}

rtst_int16
rtst_urand_i16i16_i16(rtst_int16 min, rtst_int16 max)
{
    rtst_int16 res;
    rtst_int32 range;

    assert(max>=min);
    range = (rtst_int32) max - (rtst_int32) min + 1;
    res = (rtst_int16) ((nrand48(rtst_state48) % range) + min);
    return res;
}

rtst_uint16
rtst_urand_u16u16_u16(rtst_uint16 min, rtst_uint16 max)
{
    rtst_uint16 res;
    rtst_int32 range;

    assert(max>=min);
    range = (rtst_int32) max - (rtst_int32) min + 1;
    res = (rtst_uint16) ((nrand48(rtst_state48) % range) + min);
    return res;
}

rtst_int32
rtst_urand_i32i32_i32(rtst_int32 min, rtst_int32 max)
{
    rtst_int32 res;
    rtst_uint32 range;
    rtst_int32 rand;

    assert(max>=min);
    rand = jrand48(rtst_state48);
    if (min==RTST_INT32_MIN && max==RTST_INT32_MAX)
    {
	res = rand;
    }
    else
    {
	range = (rtst_uint32) (max - min + 1);
	res = (((rtst_uint32) rand) % range) + min;
    }
    return res;
}

rtst_uint32
rtst_urand_u32u32_u32(rtst_uint32 min, rtst_uint32 max)
{
    rtst_uint32 res;
    rtst_uint32 range;
    rtst_uint32 rand;

    assert(max>=min);
    rand = (rtst_uint32) jrand48(rtst_state48);
    if (min==0 && max==RTST_UINT32_MAX)
    {
	res = rand;
    }
    else
    {
	range = (rtst_uint32) (max - min + 1);
	res = (rand % range) + min;
    }
    return res;
}

double
rtst_urand_dd_d(double min, double max)
{
    const double range = max - min;

    return ((double) erand48(rtst_state48) * range) + min;
}

void
rtst_urand_i8i8_vi8(size_t len, rtst_int8 min, rtst_int8 max,
                       rtst_int8* vec)
{
    size_t i;

    for (i=0; i<len; i++)
	*vec++ = rtst_urand_i8i8_i8(min, max);
}

void
rtst_urand_u8u8_vu8(size_t len, rtst_uint8 min, rtst_uint8 max,
                       rtst_uint8* vec)
{
    size_t i;

    for (i=0; i<len; i++)
	*vec++ = rtst_urand_u8u8_u8(min, max);
}

void
rtst_urand_i16i16_vi16(size_t len, rtst_int16 min, rtst_int16 max,
                       rtst_int16* vec)
{
    size_t i;

    for (i=0; i<len; i++)
	*vec++ = rtst_urand_i16i16_i16(min, max);
}

void
rtst_urand_u16u16_vu16(size_t len, rtst_uint16 min, rtst_uint16 max,
                       rtst_uint16* vec)
{
    size_t i;

    for (i=0; i<len; i++)
	*vec++ = rtst_urand_u16u16_u16(min, max);
}

void
rtst_urand_i32i32_vi32(size_t len,
                       rtst_int32 min, rtst_int32 max, rtst_int32* vec)
{
    size_t i;

    for (i=0; i<len; i++)
	*vec++ = rtst_urand_i32i32_i32(min, max);
}

void
rtst_urand_u32u32_vu32(size_t len,
                       rtst_uint32 min, rtst_uint32 max, rtst_uint32* vec)
{
    size_t i;

    for (i=0; i<len; i++)
	*vec++ = rtst_urand_u32u32_u32(min, max);
}

void
rtst_urand_ff_vf(size_t n, float min, float max, float *vec)
{
    size_t i;
    for (i=0; i<n; i++)
        *vec++ = rtst_urand_ff_f(min, max);
}

void
rtst_urand_dd_vd(size_t n, double min, double max, double *vec)
{
    size_t i;
    for (i=0; i<n; i++)
        *vec++ = rtst_urand_dd_d(min, max);
}

/*
  Fill floating point vectors with uniform random integers in a given
  range.
*/

void
rtst_urandi_i32i32_vf(size_t n, rtst_int32 min, rtst_int32 max, float *vec)
{
    size_t i;
    for (i=0; i<n; i++)
        vec[i] = (float) rtst_urand_i32i32_i32(min, max);
}

void
rtst_urandi_i32i32_vd(size_t n, rtst_int32 min, rtst_int32 max, double *vec)
{
    size_t i;
    for (i=0; i<n; i++)
        vec[i] = (double) rtst_urand_i32i32_i32(min, max);
}

/**********************************************************************
** Functions that produce random values biased towards "bad" cases.
*/

/* A type for generating biased random integers */

enum RAND_INT_TYPE
{
    ZERO,
    P1,
    M1,
    POW2MINUS,
    POW2,
    POW2PLUS,
    MAXMINUS,
    MAX,
    MIN,
    MINPLUS,
    OTHER,
    NUM_RAND_INT_TYPES
};

/*
** A biased 16 bit signed random number generator
**
** Generates more 0s, 1s, numbers near powers of two, maximums and
** minimums.
*/

/* TODO: Should change distribution so more uniform random numbers are */
/* generated, and should merge P1,M1 into powers of 2 cases. */

rtst_int16
rtst_badrand_i16(void)
{
    enum RAND_INT_TYPE type;    /* Type of random number */
    unsigned long r;            /* Random 31 bit positive number */
    rtst_int16 res;             /* Result */

/* We try and use the one random number as much as possible -
   statistically more dangerous, but rand48 is slow on simulators.
   Must make sure NUM_RAND_INT_TYPES != power of 2.
*/

    r = (unsigned long) nrand48(rtst_state48);
    type = (enum RAND_INT_TYPE) (r % NUM_RAND_INT_TYPES);

    switch(type)
    {
    case ZERO:
        return 0;
    case P1:
        return 1;
    case M1:
        return -1;
    case POW2MINUS:
        res = (rtst_int16) ((1 << (((r>>16) % 14) + 1)) - 1);
        if (r & 0x01000000)
            res = -res;
        return res;
    case POW2:
        res = (rtst_int16) (1 << (((r>>16) % 14) + 1));
        if (r & 0x01000000)
            res = -res;
        return res;
    case POW2PLUS:
        res = (rtst_int16) ((1 << (((r>>16) % 14) + 1)) + 1);
        if (r & 0x01000000)
            res = -res;
        return res;
    case MAXMINUS:
        return RTST_INT16_MAX - 1;
    case MAX:
        return RTST_INT16_MAX;
    case MIN:
        return RTST_INT16_MIN;
    case MINPLUS:
        return RTST_INT16_MIN + 1;
    case OTHER:
        return (rtst_int16) rtst_urand_i32i32_i32(RTST_INT16_MIN,
                                                  RTST_INT16_MAX);
    default:
        assert(0);
    }
    assert(0);
    return 0;
}

/*
** A biased 32 bit signed random number generator
**
** Generates more 0s, 1s, numbers near powers of two, maximums and
** minimums
*/

rtst_int32
rtst_badrand_i32(void)
{
    enum RAND_INT_TYPE type;    /* Type of random number. */
    unsigned long r;            /* Random 31 bit positive number. */
    rtst_int32 res;             /* Result. */

/* We try and use the one random number as much as possible -
   statistically more dangerous, but rand48 is slow on simulators.
   Must make sure NUM_RAND_INT_TYPES != power of 2.
*/

    r = (unsigned long) nrand48(rtst_state48);
    type = (enum RAND_INT_TYPE) (r % NUM_RAND_INT_TYPES);

    switch(type)
    {
    case ZERO:
        return 0;
    case P1:
        return 1;
    case M1:
        return -1;
    case POW2MINUS:
        res = (rtst_int32) ((1 << (((r>>16) % 30) + 1)) - 1);
        if (r & 0x01000000)
            res = -res;
        return res;
    case POW2:
        res = (rtst_int32) (1 << (((r>>16) % 30) + 1));
        if (r & 0x01000000)
            res = -res;
        return res;
    case POW2PLUS:
        res = (rtst_int32) ((1 << (((r>>16) % 30) + 1)) + 1);
        if (r & 0x01000000)
            res = -res;
        return res;
    case MAXMINUS:
        return RTST_INT32_MAX - 1;
    case MAX:
        return RTST_INT32_MAX;
    case MINPLUS:
        return RTST_INT32_MIN + 1;
    case MIN:
        return RTST_INT32_MIN;
    case OTHER:
        return jrand48(rtst_state48);
    default:
        assert(0);
    }
    assert(0);
    return 0;
}

void
rtst_badrand_vi16(size_t len, rtst_int16* vec)
{
    size_t i;
    for (i=0; i<len; i++)
        *vec++ = rtst_badrand_i16();
}

void
rtst_badrand_vi32(size_t len, rtst_int32* vec)
{
    int i;

    for (i=0; i<len; i++)
        *vec++ = rtst_badrand_i32();
}
